%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% FI:PA171 - Digital Filters
%
% topic: Image descriptors
% author: David Svoboda 
%         Masaryk University (c) 2019
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% --------------------------------------------------------------------
% 1. Fourier descriptors
% --------------------------------------------------------------------

% Try the Java application "FourierDescriptor.jar"

% ------------------------------------------------------------
% 2. Evaluate the seven invariant moments.
% ------------------------------------------------------------

% read an image
I = imread('brain.tif');
J = padarray(I,[50 50], 0, 'both');
% now, modify the original image:
% a) do nothing
A = J;
imshow(A);
% b) rotate it
B = imrotate(J,45, 'bilinear','crop');
imshow(B);
% c) flip it
C = flipud(J);
imshow(C);
% d) scale it
D = padarray(I(1:2:end,1:2:end),[114 114],0,'both');
imshow(D);
% finally, evaluate the invariant moments for each image
invmoments(A)
invmoments(B)
invmoments(C)
invmoments(D)

% ------------------------------------------------------------
% 3. Draw selected 2D Zernike polynomial
% ------------------------------------------------------------
%
% Display the Zernike polynomial Z(n=7,m=3)
%
n=7;
m=3;
x = -1:0.01:1;
[X,Y] = meshgrid(x,x);
[theta,r] = cart2pol(X,Y);
idx = r<=1;
z = zeros(size(X));
z(idx) = zernfun(n,m,r(idx),theta(idx));
imagesc(z)
colormap('default')
title('Zernike polynomial Z^7_3(r,\theta)')

% allign the size of the image with the polynomial
% in order we can perfomr pointwise multiplication
new_I = double(resample(I, [size(z,2)/size(I,2) size(z,1)/size(I,1)]));
zernike_feature = dot(new_I(:), z(:))

% --------------------------------------------------------------------
% 4. Haralick features
% --------------------------------------------------------------------

% Test, how the co-occurrence matrix is constructed in Matlab:

I = [0 3 3; 0 0 1; 2 2 1]

glcm_V = graycomatrix(I,'Offset',[1 0],'GrayLimits',[0 3],'NumLevels',4,'Symmetric',true)
glcm_H = graycomatrix(I,'Offset',[0 1],'GrayLimits',[0 3],'NumLevels',4,'Symmetric',true)
glcm_D = graycomatrix(I,'Offset',[1 1],'GrayLimits',[0 3],'NumLevels',4,'Symmetric',true)
glcm_d = graycomatrix(I,'Offset',[1 -1],'GrayLimits',[0 3],'NumLevels',4,'Symmetric',true)

glcm_avg = (glcm_H + glcm_V + glcm_D + glcm_d)

graycoprops(glcm_avg,{'contrast','homogeneity'})

% --------------------------------------------------------------------
% 5. Load all the *.PNG images, implement the basic LBP method and 
%    measure the similarity between the "test_face.png" and individual
%    images "face[1-5].png". The similarity can be measured e.g. using
%    Euclidean distance.
% --------------------------------------------------------------------



        
